/*********************************************
* Author: Eric Z. Ayers
*
* test_flicker - flicker one candle.
*
* See http://www.parallexefx.com/  APPNOTE 00001 Faux Candles
*
* This program is to test the leds connected to a ULN2003A driver
* for the Faux candles project.  Each LED is wired with a current limiting
* resistor and a 470uf capacitor.
*
*
* High signal enables the LED, Low signal turns the LED off.
*
* PB3 - LED0
* PB4 - LED1
* PB5 - LED2
* PB6 - LED3
* PA4 - LED4
* PA5 - LED5
* PA6 - LED6
*
* Debug LED
* PA7 - DEBUG_LED  (with a 220uf cap and a 1K resistor)
*
* Chip type           : AT TINY 26
* Clock frequency     : 8.000000 MHz
*
*********************************************/

/*************************************************************************
 *			Macros
 *************************************************************************/


/* Set CPU frequency for libc routines and macros */
//#define F_CPU 1000000UL  // 1 MHz
//#define F_CPU 14.7456E6
//#define F_CPU 4000000UL  // 4 MHz
//#define F_CPU 8000000UL  // 8 MHz
#define F_CPU 8.0E6  // 8MHz

#define LED0_PIN PB3
#define LED1_PIN PB4
#define LED2_PIN PB5
#define LED3_PIN PB6
#define LED4_PIN PA4
#define LED5_PIN PA5
#define LED6_PIN PA6

/*************************************************************************
 *			Include Files
 *************************************************************************/

#include <io.h>
#include <avr/delay.h>
#include <stdlib.h>

/*************************************************************************
 *			Static Data
 *************************************************************************/


/*************************************************************************
 *			Function Prototypes
 *************************************************************************/
static void delay_ms (uint16_t ms);
static void delay_us (int32_t us);
static void init_leds ( void );
static void set_all_leds (uint8_t value);
static void set_led (int8_t led_number, uint8_t value);

/*************************************************************************
 *			Functions
 *************************************************************************/

/* Delay using the avr libc delay functions */
static void delay_ms (uint16_t ms)
{
	for (;ms>0;ms--)
		_delay_ms(1);
}


static void delay_us (int32_t us)
{
	for (;us>0;us-=10)
		_delay_us(10);
}


/*
  Set the LED pins as outputs for their respective ports.w
  Turn off all LEDs.
 */
static void init_leds ( void )
{
	/* Set the pins for LEDs to be outputs */
	DDRB |= _BV(LED0_PIN) | _BV(LED1_PIN) | _BV(LED2_PIN) | _BV(LED3_PIN);
	DDRA |= _BV(LED4_PIN) | _BV(LED5_PIN) | _BV(LED6_PIN);
	
	/* Set the output values for the pins to low */
	set_all_leds (0);
}

/*
  Set all LEDS to the same value
   value:    > 0  set all LEDS on,  0 set all LEDS off
*/
static void set_all_leds (uint8_t value)
{
	if (value > 0)
	{
		PORTB |= _BV(LED0_PIN) | _BV(LED1_PIN) | _BV(LED2_PIN) |
			_BV(LED3_PIN);
		PORTA |= _BV(LED4_PIN) | _BV(LED5_PIN) | _BV(LED6_PIN);
	}
	else
	{
		PORTB &= ~(_BV(LED0_PIN) | _BV(LED1_PIN) | _BV(LED2_PIN)
			   | _BV(LED3_PIN));
		PORTA &= ~(_BV(LED4_PIN) | _BV(LED5_PIN) | _BV(LED6_PIN));
	}
}

/*
  Turn a single LED on or off
  
  led_number:  pass a number 0 - 6 to set the specified LED
  value:       > 0 Turn LED on, 0 turn LED off
*/
static void set_led
(
	int8_t led_number,
	uint8_t value
)
{
#define SET_LED(portnum, pin) 		\
	if (value > 0) {	        \
		(portnum) |= _BV((pin));   \
	} else { 			\
		(portnum) &= ~(_BV((pin)));\
	}
	
	switch (led_number)
	{
	case 0:
		SET_LED(PORTB, LED0_PIN);
		break;
	case 1:
		SET_LED(PORTB, LED1_PIN);
		break;		
	case 2:
		SET_LED(PORTB, LED2_PIN);
		break;		
	case 3:
		SET_LED(PORTB, LED3_PIN);
		break;		
	case 4:
		SET_LED(PORTA, LED4_PIN);
		break;		
	case 5:
		SET_LED(PORTA, LED5_PIN);
		break;
	case 6:
		SET_LED(PORTA, LED6_PIN);		
		break;		
       
	}
	
#undef SET_LED
}


/* main loop of the program */
int main(void)
{
	uint16_t count = 0;
	int8_t random_result;
	int8_t debug_led_status = 0;
	
	
	
	/* INITIALIZE */
	init_leds();

	/* enable PA7 as output for DEBUG LED*/
	DDRA |= _BV(PA7);

	/* Blink the DEBUG led to show reset; */

	for (count = 0;count <20;count++)
	{
		PORTA |= _BV(PA7);
		delay_ms(5);
		PORTA &= ~(_BV(PA7));
		delay_ms(20);
	}

	PORTA |= _BV(PA7);
	
	/* Simulate candle flicker on LED0
	   LED0 is connected with a 470UF capacitor
	   before the current limiting resistor.
	 */
	count = 0;
	while (1)
	{
		
		if (count >= 0xFF)
		{
			if (debug_led_status == 0)
			{
				/* turn the status light off */
				PORTA &= ~(_BV(PA7));
			}
			else
			{
				/* turn the status light on */
				PORTA |= _BV(PA7);

			}
			debug_led_status = !debug_led_status;
			count = 0;
		}
		count = count + 1;;

#define MAX_DELAY_MS 50L
		
		set_led(0, 1); // turn on led0
		random_result = (random() % MAX_DELAY_MS);
		
		// wait some random time  
		delay_ms(random_result); 
		

		/* Now turn the led off.  The capacitor will make
		   the LED slowly drain.
		*/
		set_led(0, 0); // turn led0 off
		random_result = (random() % MAX_DELAY_MS)*2;		
		delay_ms(random_result+100);
		
#undef MAX_DELAY_MS
	}
}

